@@ -30,6 +30,8 @@ use Panotools::Script::Line::Output;
 use Panotools::Script::Line::Control;
 use Panotools::Script::Line::ControlMorph;
 use Panotools::Script::Line::Variable;
+use Panotools::Script::Line::Mask;
+use Digest::MD5 'md5_hex';
 
 use File::Temp qw/ tempdir /;
 use File::Spec;
@@ -37,7 +39,7 @@ use Math::Trig qw/:radial pi great_circle_distance/;
 
 use Storable qw/ dclone /;
 
-our $VERSION = '0.24';
+our $VERSION = '0.25';
 
 our $CLEANUP = 1;
 $CLEANUP = 0 if defined $ENV{DEBUG};
@@ -69,6 +71,7 @@ sub _defaults
     $self->{output} = [];
     $self->{control} = [];
     $self->{controlmorph} = [];
+    $self->{mask} = [];
 }
 
 =pod
@@ -93,6 +96,8 @@ sub Read
     my @raw = <FILE>;
     close FILE;
 
+    $self->{md5} = md5_hex (join '', @raw);
+
     my ($volume, $directories, $file) = File::Spec->splitpath ($path);
 
     for my $line (@raw)
@@ -133,6 +138,12 @@ sub Read
             $imagemeta->Parse ($line);
             push @{$self->ImageMetadata}, $imagemeta;
         }
+        if ($line =~ /^k /)
+        {
+            my $mask = new Panotools::Script::Line::Mask;
+            $mask->Parse ($line);
+            push @{$self->Mask}, $mask;
+        }
     }
     $self->Output2Image;
     return 1;
@@ -184,6 +195,11 @@ sub Write
     {
          print FILE $controlmorph->Assemble;
     }
+    print FILE "\n# Mask lines\n";
+    for my $mask (@{$self->Mask})
+    {
+         print FILE $mask->Assemble;
+    }
     print FILE "\n# option lines\n";
     print FILE $self->Option->Assemble;
     print FILE "\n*\n";
@@ -283,6 +299,12 @@ sub ControlMorph
     $self->{controlmorph};
 }
 
+sub Mask
+{
+    my $self = shift;
+    $self->{mask};
+}
+
 =pod
 
 Rotate transform all the images in a project, angles in degrees:
@@ -496,6 +518,16 @@ sub Subset
         push @{$pto_out->{control}}, $clone;
     }
 
+    # copy masks for selected images
+    $pto_out->{mask} = [];
+    for my $mask (@{$self->{mask}})
+    {
+        next unless defined $mapping->{$mask->{i}};
+        my $clone = $mask->Clone;
+        $clone->{i} = $mapping->{$mask->{i}};
+        push @{$pto_out->{mask}}, $clone;
+    }
+
     return $pto_out;
 }
 
@@ -567,6 +599,20 @@ sub Merge
         push @{$self->Control}, $clone;
     }
 
+    # add masks
+    for my $mask (@{$b->Mask})
+    {
+        my $jump;
+        for my $self_mask (@{$self->Mask})
+        {
+            $jump = 1 if ($self_mask->{i} eq $mask->{i} and $self_mask->{p} eq $mask->{p});
+        }
+        next if $jump;
+        my $clone = $mask->Clone;
+        $clone->{i} = $mapping->{$b->Image->[$clone->{i}]->{n}};
+        push @{$self->Mask}, $clone;
+    }
+
     $self->Duplicates;
 
     for my $option (keys %{$b->Option})
@@ -980,6 +1026,42 @@ sub AngularDistance
     return $distance * 180/pi;
 }
 
+=pod
+
+Look at all photos and calculate an optimal pixel width for this panorama,
+optionally supply a scaling factor:
+
+   $width = $pto->OptimalWidth (0.7);
+
+This number is rounded up to the nearest multiple of 16 pixels.
+
+=cut
+
+sub OptimalWidth
+{
+    my $self = shift;
+    my $factor = shift || 1;
+    my $pix_radius_max = 1;
+    for (@{$self->Image})
+    {
+        my $pix_radius = $_->Radius ($self);
+        $pix_radius_max = $pix_radius if $pix_radius > $pix_radius_max;
+    }
+    my $pix_width;
+    my $rad_fov = Math::Trig::deg2rad ($self->Panorama->{v});
+    $rad_fov = 2 * Math::Trig::pi() if $rad_fov == 0;
+    if ($self->Panorama->{f} == 0)
+    {
+        return $self->Panorama->{w} if $self->Panorama->{v} >= 180;
+        $pix_width = 2 * $pix_radius_max * Math::Trig::tan ($rad_fov/2); 
+    }
+    else
+    {
+        $pix_width = $pix_radius_max * $rad_fov;
+    }
+    return int (($pix_width * $factor / 16) +1) * 16;
+}
+
 
 =head1 COPYRIGHT
 
